#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.knowledge_interfaces.optimization import Optimization
from pycatia.knowledge_interfaces.set_of_equation import SetOfEquation
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class Optimizations(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     Optimizations
                | 
                | Represents a collection of optimization features.
                | 
                | See also:
                |     Optimization
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Optimization)
        self.optimizations = com_object

    def create_constraints_satisfaction(self, i_name: str, i_comment: str, i_formula_body: str) -> SetOfEquation:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateConstraintsSatisfaction(CATBSTR iName,
                | CATBSTR iComment,
                | CATBSTR iFormulaBody) As SetOfEquation
                | 
                |     Returns a set of equations.
                | 
                |     Parameters:
                | 
                |         iName
                |             The name of the set of equations. 
                |         iComment
                |             The comment of the set of equations. 
                |         iFormulaBody
                |             The body of the set of equations " a==b+4; c ≤
                |             90".

        :param str i_name:
        :param str i_comment:
        :param str i_formula_body:
        :return: SetOfEquation
        :rtype: SetOfEquation
        """
        return SetOfEquation(self.optimizations.CreateConstraintsSatisfaction(i_name, i_comment, i_formula_body))

    def create_optimization(self) -> Optimization:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateOptimization() As Optimization
                | 
                |     Creates an empty optimization.
                |     This optimization cannot be used while its properties have not been set.

        :return: Optimization
        :rtype: Optimization
        """
        return Optimization(self.optimizations.CreateOptimization())

    def item(self, i_index: cat_variant) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Item(CATVariant iIndex) As AnyObject
                | 
                |     Retrieves an optimization using its index or its name from the
                |     Optimizations collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the item (optimization or
                |             constraintSatisfaction) to retrieve from the collection of optimizations. As a
                |             numerics, this index is the rank of the item in the collection. The index of
                |             the first item in the collection is 1, and the index of the last item is Count.
                |             As a string, it is the name you assigned to the item using the
                |             
                | 
                |         AnyObject.Name property or when changing the item name by the property
                |         panel. 
                |     Returns:
                |         either the retrieved optimization or the retreived
                |         constraintSatisfaction 
                |     Example:
                |         This example retrieves the last item (optimization or
                |         constraintSatisfaction) in the optimizations
                |         collection.
                | 
                |          Set lastItem = optimizations.Item(optimizations.Count)

        :param cat_variant i_index:
        :return: AnyObject
        :rtype: AnyObject
        """
        return AnyObject(self.optimizations.Item(i_index))

    def __getitem__(self, n: int) -> Optimization:
        if (n + 1) > self.count:
            raise StopIteration

        return Optimization(self.optimizations.item(n + 1))

    def __iter__(self) -> Iterator[Optimization]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'Optimizations(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.drafting_interfaces.drawing_leader import DrawingLeader
from pycatia.system_interfaces.collection import Collection


class DrawingLeaders(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     DrawingLeaders
                | 
                | A collection of all the drawing leaders currently managed by a drawing view of
                | drawing sheet in a drawing document.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=DrawingLeader)
        self.drawing_leaders = com_object

    def add(self, i_head_point_x: float, i_head_point_y: float) -> DrawingLeader:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Add(double iHeadPointX,
                | double iHeadPointY) As DrawingLeader
                | 
                |     Creates a drawing leader and adds it to the DrawingLeaders
                |     collection.
                | 
                |     Parameters:
                | 
                |         iHeadPointX,iHeadPointY
                |             The x and y coordinates of head side of drawing leader
                |             
                | 
                |     Returns:
                |         The created drawing leader 
                | 
                | Example:
                |     The following example creates a drawing leader and retrieved in MyLeader in
                |     the drawing text collection of the MyText drawing text. This text belongs to
                |     the drawing text collection of the drawing view
                | 
                |      Dim MyTexts As DrawingTexts
                |      Set MyTexts = MySheet.Views.ActiveView
                |      Dim MyText As DrawingText
                |      Set MyText = MyTexts.Item(1)
                |      Dim MyLeader As DrawingLeader
                |      Set MyLeader = MyText.Leaders.Add(20., 50)

        :param float i_head_point_x:
        :param float i_head_point_y:
        :return: DrawingLeader
        :rtype: DrawingLeader
        """
        return DrawingLeader(self.drawing_leaders.Add(i_head_point_x, i_head_point_y))

    def item(self, i_index: int) -> DrawingLeader:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Item(long iIndex) As DrawingLeader
                | 
                |     Returns a drawing leader using its index from the DrawingLeaders
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index of the drawing leader to retrieve from the collection of
                |             drawing arrows. As a numerics, this index is the rank of the drawing leader in
                |             the collection. The index of the first drawing leader in the collection is 1,
                |             and the index of the last drawing leader is Count.
                |             
                | 
                |     Returns:
                |         The retrieved drawing view 
                |     Example:
                |         This example retrieves in ThisDrawingLeader the second drawing leader,
                |         in the drawing view collection of the active view in the active sheet, in the
                |         active document supposed to be a drawing document.
                | 
                |          Dim MyView  As DrawingView
                |          Set MyView  = MySheet.Views.ActiveView
                |          Dim ThisDrawingLeader As DrawingLeader
                |          Set ThisDrawingLeader = MyView.Leaders.Item(2)

        :param int i_index:
        :return: DrawingLeader
        :rtype: DrawingLeader
        """
        return DrawingLeader(self.drawing_leaders.Item(i_index))

    def remove(self, i_index: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Remove(long iIndex)
                | 
                |     Removes a drawing leader from the DrawingLeaders
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index of the drawing leader to remove from the collection of
                |             drawing leaders. As a numerics, this index is the rank of the drawing leader in
                |             the collection. The index of the first drawing leader in the collection is 1,
                |             and the index of the last drawing leader is Count.
                |             
                | 
                |     Example:
                |         The following example removes the third drawing leader in the drawing
                |         leader collection of the active view of the active document, supposed to be a
                |         drawing document.
                | 
                |          Dim MyView As DrawingView
                |          Set MyView  = MySheet.Views.ActiveView
                |          MyView.DrawingLeaders.Remove(3)

        :param int i_index:
        :return: None
        :rtype: None
        """
        return self.drawing_leaders.Remove(i_index)

    def __getitem__(self, n: int) -> DrawingLeader:
        if (n + 1) > self.count:
            raise StopIteration

        return DrawingLeader(self.drawing_leaders.item(n + 1))

    def __iter__(self) -> Iterator[DrawingLeader]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'DrawingLeaders(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.mec_mod_interfaces.boundary import Boundary
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class HybridShapes(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     HybridShapes
                | 
                | The collection of the HybridShapes making up a body.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=HybridShape)
        self.hybrid_shapes = com_object

    def get_boundary(self, i_label: str) -> Boundary:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetBoundary(CATBSTR iLabel) As Boundary
                | 
                |     Returns a boundary using its label.
                | 
                |     Parameters:
                | 
                |         iLabel
                |             Identification of the 
                | 
                |         Boundary object. See Reference.DisplayName. 
                |     Returns:
                |         The retrieved boundary

        :param str i_label:
        :return: Boundary
        :rtype: Boundary
        """
        return Boundary(self.hybrid_shapes.GetBoundary(i_label))

    def item(self, i_index: cat_variant) -> HybridShape:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iIndex) As HybridShape
                | 
                |     Returns a HybridShape using its index or its name from the HybridShapes
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the HybridShape to retrieve from the
                |             collection of HybridShapes. As a numerics, this index is the rank of the
                |             HybridShape in the collection. The index of the first HybridShape in the
                |             collection is 1, and the index of the last HybridShape is
                |             
                | 
                |         Collection.Count. As a string, it is the name you assigned to the
                |         HybridShape using the AnyObject.Name property. 
                |     Returns:
                |         The retrieved HybridShape 
                |     Example:
                |         This example retrieves in ThisHybridShape the third HybridShape, and in
                |         ThatHybridShape the HybridShape named MyHybridShape in the HybridShape
                |         collection of the active document, supposed to be a part
                |         document.
                | 
                |          Set ThisHybridShape = CATIA.ActiveDocument.HybridShapes.Item(3)
                |          Set ThatHybridShape = CATIA.ActiveDocument.HybridShapes.Item("MyHybridShape")

        :param cat_variant i_index:
        :return: HybridShape
        :rtype: HybridShape
        """
        return HybridShape(self.hybrid_shapes.Item(i_index))

    def __getitem__(self, n: int) -> HybridShape:
        if (n + 1) > self.count:
            raise StopIteration

        return HybridShape(self.hybrid_shapes.item(n + 1))

    def __iter__(self) -> Iterator[HybridShape]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'HybridShapes(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.hybrid_shape_direction import HybridShapeDirection
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeReflectLine(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeReflectLine
                | 
                | Represents the hybrid shape reflect line feature object.
                | Role: To access the data of the hybrid shape reflect line feature object. This
                | data includes:
                | 
                |     The surface used to create the reflect line
                |     The direction (cylindrical)
                |     The origin (conical)
                |     The angle value
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeReflectLine
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_reflect_line = com_object

    @property
    def angle(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Angle() As Angle
                | 
                |     Returns or sets the angle used to create the reflectline.
                | 
                |     Example:
                |         This example retrieves in Ang the angle for the RelectLine hybrid shape
                |         feature.
                | 
                |          Dim Ang As CATIAAngle
                |          Set Ang = ReflectLine.Angle

        :return: Angle
        :rtype: Angle
        """

        return Angle(self.hybrid_shape_reflect_line.Angle)

    @angle.setter
    def angle(self, angle: Angle):
        """
        :param Angle angle:
        """

        self.hybrid_shape_reflect_line.Angle = angle.com_object

    @property
    def direction(self) -> HybridShapeDirection:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Direction() As HybridShapeDirection
                | 
                |     Returns or sets the direction used to create the cylindrical
                |     reflectline.
                | 
                |     Example:
                |         This example retrieves in Dir the direction for the cylindrical
                |         RelectLine hybrid shape feature.
                | 
                |          Dim Dir As CATIAHybridShapeDirection
                |          Set Dir = ReflectLine.Direction

        :return: HybridShapeDirection
        :rtype: HybridShapeDirection
        """

        return HybridShapeDirection(self.hybrid_shape_reflect_line.Direction)

    @direction.setter
    def direction(self, direction: HybridShapeDirection):
        """
        :param HybridShapeDirection direction:
        """

        self.hybrid_shape_reflect_line.Direction = direction.com_object

    @property
    def orientation_direction(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OrientationDirection() As long
                | 
                |     Returns or sets the direction orientation used to compute the reflect
                |     line.
                |     Role: The orientation is used to define the angle between the direction and
                |     the normal to the support of the points on the result curve. The orientation is
                |     the same than or the inverse of the result of the cross product:
                |     Normal(support) ^ Tangent(FirstReferenceCurve).
                |     Legal values: 1 for same orientation, and -1 for inverse

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_reflect_line.OrientationDirection

    @orientation_direction.setter
    def orientation_direction(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_reflect_line.OrientationDirection = value

    @property
    def orientation_support(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OrientationSupport() As long
                | 
                |     Returns or sets the support orientation used to compute the reflect
                |     line.
                |     Role: The orientation is used to define the angle between the direction and
                |     the normal to the support of the points on the result curve. The orientation is
                |     the same than or the inverse of the result of the cross product:
                |     Normal(support) ^ Tangent(FirstReferenceCurve).
                |     Legal values: 1 for same orientation, and -1 for inverse

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_reflect_line.OrientationSupport

    @orientation_support.setter
    def orientation_support(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_reflect_line.OrientationSupport = value

    @property
    def origin(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Origin() As Reference
                | 
                |     Returns or sets the origin point used to create the conical
                |     reflectline.
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example retrieves in Point the origin point for the conical
                |         ReflectLine hybrid shape feature.
                | 
                |          Dim Point As Reference
                |          Set Point = ReflectLine.Origin

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_reflect_line.Origin)

    @origin.setter
    def origin(self, reference_origin: Reference):
        """
        :param Reference reference_origin:
        """

        self.hybrid_shape_reflect_line.Origin = reference_origin.com_object

    @property
    def source_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SourceType() As long
                | 
                |     Returns or sets whether the reflectline curve is or should be created with
                |     infinite light source (cylindrical) or with finite point light source
                |     (conical).
                |     Role: The SourceType indicates whether the created reflectline curve is
                |     compute with infinite light source for cylindrical type or with finite point
                |     light source for conical type.
                |     Legal values: 0 for cylindrical and 1 for conical.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_reflect_line.SourceType

    @source_type.setter
    def source_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_reflect_line.SourceType = value

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Returns or sets the support surface used to create the
                |     reflectline.
                |     Sub-element(s) supported (see Boundary object): Face.
                | 
                |     Example:
                |         This example retrieves in Surface the support surface for the
                |         RelectLine hybrid shape feature.
                | 
                |          Dim Surface As Reference
                |          Set Surface = ReflectLine.Support

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_reflect_line.Support)

    @support.setter
    def support(self, reference_support: Reference):
        """
        :param Reference reference_support:
        """

        self.hybrid_shape_reflect_line.Support = reference_support.com_object

    @property
    def type_solution(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property TypeSolution() As long
                | 
                |     Returns or sets whether the reflectline curve is or should be created with
                |     the normal to the support or the tangent plane to the
                |     support.
                |     Role: The TypeSolution indicates whether the created reflectline curve is
                |     compute with the angle between the normale to the support and the direction or
                |     with the angle between the tangent plane to the support and the
                |     direction.
                |     Legal values: 0 for the normal and 1 for the tangent plane.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_reflect_line.TypeSolution

    @type_solution.setter
    def type_solution(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_reflect_line.TypeSolution = value

    def invert_orientation_direction(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InvertOrientationDirection()
                | 
                |     Inverts the orientation of direction. This example inverts the direction
                |     orientation of hybRefLine hybrid shape reflect line
                |     object.
                | 
                |      hybRefLine.InvertOrientationDirection

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_reflect_line.InvertOrientationDirection()

    def invert_orientation_support(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InvertOrientationSupport()
                | 
                |     Inverts the orientation of support. This example inverts the support
                |     orientation of hybRefLine hybrid shape reflect line
                |     object.
                | 
                |      hybRefLine.InvertOrientationSupport

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_reflect_line.InvertOrientationSupport()

    def __repr__(self):
        return f'HybridShapeReflectLine(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.part_interfaces.draft_domains import DraftDomains
from pycatia.part_interfaces.dress_up_shape import DressUpShape


class Draft(DressUpShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Shape
                |                         PartInterfaces.DressUpShape
                |                             Draft
                | 
                | Represents the draft shape.
                | A draft shape is made up of draft domains (at least one) and of a parting
                | element.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.draft = com_object

    @property
    def draft_domains(self) -> DraftDomains:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property DraftDomains() As DraftDomains (Read Only)
                | 
                |     Returns the collection of draft domains.
                | 
                |     Example:
                |         The following example returns in list the collection of draft domains
                |         of the firstDraft draft:
                | 
                |          Set list = firstDraft.DraftDomains

        :return: DraftDomains
        :rtype: DraftDomains
        """

        return DraftDomains(self.draft.DraftDomains)

    @property
    def mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Mode() As CatDraftMode
                | 
                |     Returns or sets the draft mode.
                | 
                |     Example:
                |         The following example returns in mode the draft mode of the firstDraft
                |         draft, and then sets it to
                |         CatReflectKeepFaceDraftMode:
                | 
                |          Set mode = firstDraft.Mode
                |          Set firstDraft.Mode = CatReflectKeepFaceDraftMode

        :return: int
        :rtype: int
        """

        return self.draft.Mode

    @mode.setter
    def mode(self, value: int):
        """
        :param int value:
        """

        self.draft.Mode = value

    @property
    def parting_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PartingElement() As Reference
                | 
                |     Returns or sets the draft parting element.
                |     To set the property, you can use the following Boundary object:
                |     PlanarFace.
                | 
                |     Example:
                |         The following example returns in element the parting element of the
                |         firstDraft draft, and then sets it to the element2 geometrical
                |         element:
                | 
                |          Set element = firstDraft.PartingElement
                |          Set firstDraft.PartingElement = element2

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.draft.PartingElement)

    @parting_element.setter
    def parting_element(self, value: Reference):
        """
        :param Reference value:
        """

        self.draft.PartingElement = value

    def __repr__(self):
        return f'Draft(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeOffset(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeOffset
                | 
                | Represents the hybrid shape offset feature object.
                | Role: To access the data of the hybrid shape offset feature object. This data
                | includes:
                | 
                |     The element to offset
                |     The offset direction
                |     The offset value
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeOffset
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_offset = com_object

    @property
    def offset_direction(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OffsetDirection() As boolean
                | 
                |     Returns or sets whether the offset direction is to be
                |     inverted.
                |     True to invert the offset direction.

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_offset.OffsetDirection

    @offset_direction.setter
    def offset_direction(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_offset.OffsetDirection = value

    @property
    def offset_value(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OffsetValue() As Length
                | 
                |     Returns or sets the offset value.

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_offset.OffsetValue)

    @offset_value.setter
    def offset_value(self, length: Length):
        """
        :param Length length:
        """

        self.hybrid_shape_offset.OffsetValue = length.com_object

    @property
    def offseted_object(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OffsetedObject() As Reference
                | 
                |     Returns or sets the face to offset.
                |     Sub-element(s) supported (see Boundary object): Face.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_offset.OffsetedObject)

    @offseted_object.setter
    def offseted_object(self, reference_object: Reference):
        """
        :param Reference reference_object:
        """

        self.hybrid_shape_offset.OffsetedObject = reference_object.com_object

    @property
    def suppress_mode(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SuppressMode() As boolean
                | 
                |     Returns or sets suppress mode.
                |     True to activate suppress mode

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_offset.SuppressMode

    @suppress_mode.setter
    def suppress_mode(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_offset.SuppressMode = value

    def add_tricky_face(self, i_tricky_face: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddTrickyFace(Reference iTrickyFace)
                | 
                |     Adds a tricky face object on the object.

        :param Reference i_tricky_face:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_offset.AddTrickyFace(i_tricky_face.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_tricky_face'
        # # vba_code = """
        # # Public Function add_tricky_face(hybrid_shape_offset)
        # #     Dim iTrickyFace (2)
        # #     hybrid_shape_offset.AddTrickyFace iTrickyFace
        # #     add_tricky_face = iTrickyFace
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_tricky_face(self, i_rank: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetTrickyFace(long iRank) As Reference
                | 
                |     Returns the invalid face object on the object.
                |     param : iRank =position of faces ionvalid for offset

        :param int i_rank:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.hybrid_shape_offset.GetTrickyFace(i_rank))

    def remove_tricky_face(self, i_rank: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveTrickyFace(long iRank)
                | 
                |     Remove the tricky face object on the object.
                |     param : iRank =position of the face in the list of TrickyFaces

        :param int i_rank:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_offset.RemoveTrickyFace(i_rank)

    def set_offset_value(self, i_offset: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetOffsetValue(double iOffset)
                | 
                |     Set Offset value with input as double.
                |     To be replace in further release by integration in the pragma put_xxx

        :param float i_offset:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_offset.SetOffsetValue(i_offset)

    def __repr__(self):
        return f'HybridShapeOffset(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.line import Line
from pycatia.in_interfaces.reference import Reference


class HybridShapeLineBiTangent(Line):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         CATGSMIDLItf.Line
                |                             HybridShapeLineBiTangent
                | 
                | Line tangent to a curve.
                | Role: To access data of the line feature created to be tangent to two
                | entities
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_line_bi_tangent = com_object

    @property
    def curve1(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Curve1() As Reference
                | 
                |     Returns or Sets the first tangency curve lying on the support
                |     surface.
                |     Sub-element(s) supported (see Boundary object): TriDimFeatEdge or
                |     BiDimFeatEdge.
                | 
                |     Example:
                |         This example retrieves in oCurve the first tangency curve for the
                |         LineBiTangent hybrid shape feature.
                | 
                |          Dim oCurve As Reference
                |          Set oCurve = LineBiTangent.Curve1

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_bi_tangent.Curve1)

    @curve1.setter
    def curve1(self, reference_curve: Reference):
        """
        :param Reference reference_curve:
        """

        self.hybrid_shape_line_bi_tangent.Curve1 = reference_curve.com_object

    @property
    def element2(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Element2() As Reference
                | 
                |     Returns or Sets the second tangency element (point, curve) lying on the
                |     support surface.
                |     Sub-element(s) supported (see Boundary object): TriDimFeatEdge,
                |     BiDimFeatEdge or Vertex.
                | 
                |     Example:
                |         This example retrieves in oElement the second tangency Element (point,
                |         curve) for the LineBiTangent hybrid shape feature.
                | 
                |          Dim oElement As Reference
                |          Set oElement = LineBiTangent.Element2

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_bi_tangent.Element2)

    @element2.setter
    def element2(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_line_bi_tangent.Element2 = reference_element.com_object

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Returns or Sets the supporting surface.
                |     Sub-element(s) supported (see Boundary object): Face.
                | 
                |     Example:
                |         This example retrieves in oSurface the surface for the LineBiTangent
                |         hybrid shape feature.
                | 
                |          Dim oSurface As Reference 
                |          Set oSurface = LineBiTangent.Surface

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_bi_tangent.Support)

    @support.setter
    def support(self, reference_support: Reference):
        """
        :param Reference reference_support:
        """

        self.hybrid_shape_line_bi_tangent.Support = reference_support.com_object

    def get_choice_no(self, val1: int, val2: int, val3: int, val4: int, val5: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetChoiceNo(long val1,
                | long val2,
                | long val3,
                | long val4,
                | long val5)
                | 
                |     Returns a sequence which identifies a solution among all
                |     possibilities.
                | 
                |         val1 = Solution number (from 1 to n).
                | 
                |     val2 = oOriTgt1
                | 
                |     This orientation allows to compute just the results that are tangent to a
                |     specific region of the first curve.
                |     It can take 3 values:
                | 
                |         +1 : the result has the same orientation as the curve,
                |         -1 : the result has the opposite orientation of the curve,
                |         0 : no orientation is specified.
                | 
                |     val3 = oOriCvt1
                | 
                |     This orientation allows to compute just the results that are tangent to a
                |     specific side of the first curve.
                |     It can take 3 values:
                | 
                |         +1 : curvature direction of curve and cross product of support normal and result direction
                |              are in the same direction,
                |         -1 : curvature direction of curve and cross product of support normal and result direction
                |              are in opposite directions,
                |         0 : no orientation is specified.
                | 
                |     val4 = oOriTgt2
                | 
                |     In case of curve/curve bitangent line, this orientation allows to compute
                |     just the results that are tangent to a specific region of the second curve
                |     .
                |     It can take 3 values:
                | 
                |         +1 : the result has the same orientation as the curve,
                |         -1 : the result has the opposite orientation of the curve,
                |         0 : no orientation is specified.
                | 
                |     val5 = oOriCvt2
                | 
                |     In case of curve/curve bitangent line this orientation allows to compute
                |     just the results that are tangent to a specific side of the second
                |     curve.
                |     It can take 3 values:
                | 
                |         +1 : curvature direction of curve and cross product of support normal and result direction
                |              are in the same direction,
                |         -1 : curvature direction of curve and cross product of support normal and result direction
                |              are in opposite directions,
                |         0 : no orientation is specified.
                | 
                |     Example:
                |         This example retrieves in vakl1,val2,val3,val4,val5 parameters for
                |         solutions for the LineBiTangent hybrid shape feature.
                | 
                |          Dim oVal1 As long
                |          Dim oVal2 As long
                |          Dim oVal3 As long
                |          Dim oVal4 As long
                |          Dim oVal5 As long
                |          LineBiTangent.GetChoiceNo(ovla1, oVal2, oVal3, oVal4, oVal5)

        :param int val1:
        :param int val2:
        :param int val3:
        :param int val4:
        :param int val5:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_bi_tangent.GetChoiceNo(val1, val2, val3, val4, val5)

    def get_length_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetLengthType() As long
                | 
                |     Gets the length type Default is 0.
                | 
                |     Parameters:
                | 
                |         oType
                |             The length type = 0 : length - the line is limited by its extremities =
                |             1 : infinite - the line is infinite = 2 : infinite start point - the line is infinite on
                |             the side of the start point = 3 : infinite end point - the line is infinite on the side
                |             of the end point

        :return: int
        :rtype: int
        """
        return self.hybrid_shape_line_bi_tangent.GetLengthType()

    def set_choice_no(self, val1: int, val2: int, val3: int, val4: int, val5: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetChoiceNo(long val1,
                | long val2,
                | long val3,
                | long val4,
                | long val5)
                | 
                |     Sets a sequence which identifies a solution among all
                |     possibilities.
                | 
                |         val1 = Solution number (from 1 to n).
                | 
                |     val2 = oOriTgt1
                | 
                |     This orientation allows to compute just the results that are tangent to a
                |     specific region of the first curve.
                |     It can take 3 values:
                | 
                |         +1 : the result has the same orientation as the curve,
                |         -1 : the result has the opposite orientation of the curve,
                |         0 : no orientation is specified.
                | 
                |     val3 = oOriCvt1
                | 
                |     This orientation allows to compute just the results that are tangent to a
                |     specific side of the first curve.
                |     It can take 3 values:
                | 
                |         +1 : curvature direction of curve and cross product of support normal and result direction
                |              are in the same direction,
                |         -1 : curvature direction of curve and cross product of support normal and result direction
                |              are in opposite directions,
                |         0 : no orientation is specified.
                | 
                |     val4 = oOriTgt2
                | 
                |     In case of curve/curve bitangent line, this orientation allows to compute
                |     just the results that are tangent to a specific region of the second curve
                |     .
                |     It can take 3 values:
                | 
                |         +1 : the result has the same orientation as the curve,
                |         -1 : the result has the opposite orientation of the curve,
                |         0 : no orientation is specified.
                | 
                |     val5 = oOriCvt2
                | 
                |     In case of curve/curve bitangent line this orientation allows to compute
                |     just the results that are tangent to a specific side of the second
                |     curve.
                |     It can take 3 values:
                | 
                |         +1 : curvature direction of curve and cross product of support normal and result direction
                |              are in the same direction,
                |         -1 : curvature direction of curve and cross product of support normal and result direction
                |              are in opposite directions,
                |         0 : no orientation is specified.
                | 
                |     Example:
                |         This example retrieves in vakl1,val2,val3,val4,val5 parameters for
                |         solutions for the LineBiTangent hybrid shape feature.
                | 
                |          Dim iVal1 As long
                |          Dim iVal2 As long
                |          Dim iVal3 As long
                |          Dim iVal4 As long
                |          Dim iVal5 As long
                |          ival1 = 1 
                |          ival2 = 0 
                |          ival3 = 0 
                |          ival4 = 0 
                |          ival5 = 0 
                |          LineBiTangent.SetChoiceNo(ivla1, iVal2, iVal3, iVal4, iVal5)

        :param int val1:
        :param int val2:
        :param int val3:
        :param int val4:
        :param int val5:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_bi_tangent.SetChoiceNo(val1, val2, val3, val4, val5)

    def set_length_type(self, i_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLengthType(long iType)
                | 
                |     Sets the length type Default is 0.
                | 
                |     Parameters:
                | 
                |         iType
                |             The length type = 0 : length - the line is limited by its extremities =
                |             1 : infinite - the line is infinite = 2 : infinite start point - the line is infinite
                |             on the side of the start point = 3 : infinite end point - the line is infinite on the
                |             side of the end point

        :param int i_type:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_bi_tangent.SetLengthType(i_type)

    def __repr__(self):
        return f'HybridShapeLineBiTangent(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.str_param import StrParam
from pycatia.part_interfaces.dress_up_shape import DressUpShape


class Thread(DressUpShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Shape
                |                         PartInterfaces.DressUpShape
                |                             Thread
                | 
                | Represents the Thread feature.
                | It threads or taps cylindrical surface .
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.thread = com_object

    @property
    def depth(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Depth() As double
                | 
                |     Returns the thread/tap depth.
                | 
                |     Returns:
                |         oDepth Value of the thread/tap depth
                | 
                |         Example:
                |             The following example returns in Depth the depth of thread
                |             firstthread:
                | 
                |              Set Depth = firstthread.Depth

        :return: float
        :rtype: float
        """

        return self.thread.Depth

    @depth.setter
    def depth(self, value: float):
        """
        :param float value:
        """

        self.thread.Depth = value

    @property
    def diameter(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Diameter() As double
                | 
                |     Returns the thread/tap diameter.
                | 
                |     Returns:
                |         oDiameter Value of the thread/tap diameter
                | 
                |         Example:
                |             The following example returns in ThreadDiameter the diameter of
                |             thread firstthread:
                | 
                |              Set ThreadDiameter = firstthread.Diameter

        :return: float
        :rtype: float
        """

        return self.thread.Diameter

    @diameter.setter
    def diameter(self, value: float):
        """
        :param float value:
        """

        self.thread.Diameter = value

    @property
    def lateral_face_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property LateralFaceElement() As Reference
                | 
                |     Returns or sets the lateral face (must be cylindrical) .
                |     To set the property, you can use the following Boundary object: Face.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.thread.LateralFaceElement)

    @lateral_face_element.setter
    def lateral_face_element(self, value: Reference):
        """
        :param Reference value:
        """

        self.thread.LateralFaceElement = value

    @property
    def limit_face_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property LimitFaceElement() As Reference
                | 
                |     Returns or sets the limit face (must be planar ) .
                |     To set the property, you can use the following Boundary object: PlanarFace.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.thread.LimitFaceElement)

    @limit_face_element.setter
    def limit_face_element(self, value: Reference):
        """
        :param Reference value:
        """

        self.thread.LimitFaceElement = value

    @property
    def pitch(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Pitch() As double
                | 
                |     Returns the thread/tap pitch.
                | 
                |     Returns:
                |         oPitch Value of the thread/tap pitch
                | 
                |         Example:
                |             The following example returns in ThreadPitch the thread pitch of
                |             thread firstthread:
                | 
                |              Set ThreadPitch = firstthread.ThreadPitch

        :return: float
        :rtype: float
        """

        return self.thread.Pitch

    @pitch.setter
    def pitch(self, value: float):
        """
        :param float value:
        """

        self.thread.Pitch = value

    @property
    def side(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Side() As CatThreadSide
                | 
                |     Returns the thread or tap side.
                | 
                |     Returns:
                |         oThreadSide The thread/tap side (see CatThreadSide for list of possible
                |         sides)
                | 
                |         Example:
                |             The following example returns in ThreadSide the thread/tap side of
                |             thread firstthread:
                | 
                |              Set ThreadSide = firstthreadoThreadSide

        :return: int
        :rtype: int
        """

        return self.thread.Side

    @side.setter
    def side(self, value: int):
        """
        :param int value:
        """

        self.thread.Side = value

    @property
    def thread_description(self) -> StrParam:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ThreadDescription() As StrParam (Read Only)
                | 
                |     Returns the thread/tap description parameter. This call is valid only when
                |     a standard/user design table created
                | 
                |     Returns:
                |         oThreadDescParam A Parameter object controlling the thread/tap
                |         description (see StrParam for more information)
                | 
                |         Example:
                |             The following example returns in threadDescription the thread
                |             description (M12 etc) of thread firstthread:
                | 
                |              Set threadDescription = firstthread.ThreadDescription

        :return: StrParam
        :rtype: StrParam
        """

        return StrParam(self.thread.ThreadDescription)

    def create_standard_thread_design_table(self, i_standard_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub CreateStandardThreadDesignTable(CatThreadStandard
                | iStandardType)
                | 
                |     Creates a Standard Thread design table .
                | 
                |     Parameters:
                | 
                |         iStandardType
                |             Standard type for thread (see 
                | 
                |         CatThreadStandard for list of possible types)
                | 
                |         Example:
                |             The following example creates a standard table for MetricThinPitch
                |             for thread firstthread:
                | 
                |              firstthread.CreateStandardThreadDesignTable
                |              catMetricThinPitch

        :param int i_standard_type:
        :return: None
        :rtype: None
        """
        return self.thread.CreateStandardThreadDesignTable(i_standard_type)

    def create_user_standard_design_table(self, i_standard_name: str, i_path: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub CreateUserStandardDesignTable(CATBSTR iStandardName,
                | CATBSTR iPath)
                | 
                |     Creates a UserStandard Thread design table .
                | 
                |     Parameters:
                | 
                |         iStandardName
                |             Name of the UserStandard thread. iStandardName should be empty if
                |             filepath is to be defined. 
                |         iPath
                |             Path of the UserStandard file. iPath is empty if the filepath is
                |             already defined through CATReffilesPath.
                | 
                |             Example1:
                |                 The following example creates a standard table for UserStandard
                |                 for thread firstThread. The file path is already defined thru
                |                 CATReffilesPath:
                | 
                |                  firstThread.CreateUserStandardDesignTable
                |                  "UserStandard",""
                | 
                |             Example2:
                |                 The following example creates a standard table for UserStandard
                |                 for thread firstThread when file path is not defined thru
                |                 CATReffilesPath:
                | 
                |                  firstThread.CreateUserStandardDesignTable
                |                  "","E:\\user\\standard\\UserStandard.txt"

        :param str i_standard_name:
        :param str i_path:
        :return: None
        :rtype: None
        """
        return self.thread.CreateUserStandardDesignTable(i_standard_name, i_path)

    def reverse_direction(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub ReverseDirection()
                | 
                |     Swap the direction of the thread or the tap.

        :return: None
        :rtype: None
        """
        return self.thread.ReverseDirection()

    def set_explicit_polarity(self, i_thread_polarity: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetExplicitPolarity(CatThreadPolarity iThreadPolarity)
                | 
                |     Sets the thread polarity explicit. Thread polarity is no more evaluated
                |     implicitly on basis of support face polarity
                | 
                |     Parameters:
                | 
                |         iThreadPolarity
                |             Standard type for thread (see 
                | 
                |         CatThreadPolarity for list of possible types)
                | 
                |         Example:
                |             The following example sets the thread polarity to Tap explicitly
                |             thread firstthread:
                | 
                |              firstthread.SetExplicitPolarity catTap

        :param int i_thread_polarity:
        :return: None
        :rtype: None
        """
        return self.thread.SetExplicitPolarity(i_thread_polarity)

    def __repr__(self):
        return f'Thread(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from typing import TYPE_CHECKING

from pycatia.knowledge_interfaces.free_parameter import FreeParameter
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant

if TYPE_CHECKING:
    from pycatia.knowledge_interfaces.real_param import RealParam


class FreeParameters(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     FreeParameters
                | 
                | Interface to access a CATIAFreeParameters.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.free_parameters = com_object

    def add_free_parameter(self, parameter: 'RealParam') -> FreeParameter:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func AddFreeParameter(RealParam parameter) As
                | FreeParameter
                | 
                |     Adds a free parameter. This parameter must not be read only.

        :param RealParam parameter:
        :return: FreeParameter
        :rtype: FreeParameter
        """
        return FreeParameter(self.free_parameters.AddFreeParameter(parameter.com_object))

    def item(self, i_index: cat_variant) -> FreeParameter:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Item(CATVariant iIndex) As FreeParameter
                | 
                |     Retrieves an optimization using its index or its name from the Free
                |     Parameters collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the free parameter to retrieve from the
                |             collection of free parameters. As a numerics, this index is the rank of the
                |             free parameter in the collection. The index of the first free parameter in the
                |             collection is 1, and the index of the last free parameter is Count. As a
                |             string, it is the name you assigned to the free parameter using the
                |             
                | 
                |         AnyObject.Name property or when changing the free parameter name by the
                |         property panel. 
                |     Returns:
                |         The retrieved free parameter 
                |     Example:
                |         This example retrieves the last free parameter in the free parameters
                |         collection.
                | 
                |          Set lastFreeParameter = freeParameters.Item(freeParameters.Count)

        :param cat_variant i_index:
        :return: FreeParameter
        :rtype: FreeParameter
        """
        return FreeParameter(self.free_parameters.Item(i_index))

    def remove_free_parameter(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub RemoveFreeParameter(CATVariant iIndex)
                | 
                |     Removes a free parameter.

        :param cat_variant i_index:
        :return: None
        """
        return self.free_parameters.RemoveFreeParameter(i_index)

    def __getitem__(self, n: int) -> FreeParameter:
        if (n + 1) > self.count:
            raise StopIteration

        return FreeParameter(self.free_parameters.item(n + 1))

    def __iter__(self) -> Iterator[FreeParameter]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'FreeParameters(name="{self.name}")'



import logging

import sys


# the following code was shamefully pilfered from Flask's logging.


def has_level_handler(logger: logging.Logger):
    """

    Check if there is a handler in the logging chain that will handle the
    given logger's :meth:`effective level <~logging.Logger.getEffectiveLevel>`

    :param logger: logging.Logger
    :return:
    """
    """
        Check if there is a handler in the logging chain that will handle the
        given logger's :meth:`effective level <~logging.Logger.getEffectiveLevel>`.
    
    """
    level = logger.getEffectiveLevel()
    current = logger

    while current:
        if any(handler.level <= level for handler in current.handlers):
            return True

        if not current.propagate:
            break

        current = current.parent

    return False


def create_logger() -> logging.Logger:
    """

    :return: Logger
    :rtype: logging.Logger
    """

    #: ``[%(asctime)s] %(levelname)s in %(module)s: %(message)s``.
    default_handler = logging.StreamHandler(sys.stderr)
    default_handler.setFormatter(
        logging.Formatter("[%(asctime)s] %(levelname)s in %(module)s: %(message)s")
    )

    logger = logging.getLogger('pycatia')

    logger.setLevel(logging.INFO)
    
    # clear existed Handlers
    logger.handlers.clear()
    
    logger.addHandler(default_handler)

    return logger



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.in_interfaces.camera import Camera
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class Cameras(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     Cameras
                | 
                | A collection of all the Camera objects currently attached to a Document
                | object.
                | A camera can be created using the Viewer.NewCamera method of the Viewer object.
                | The first seventh cameras of the collection are Camera3D objects and cannot be
                | modified or removed. They can just be retrieved and used "as is". They store
                | the following viewpoints whose sight direction is always toward the 3D-axis
                | system origin:
                | 
                | * iso
                |     The origin is on a line with (1,1,1) as components with positive
                |     coordinates 
                | * front
                |     The origin is on the x axis with a positive x coordinate 
                | * back
                |     The origin is on the x axis with a negative x coordinate 
                | * left
                |     The origin is on the y axis with a positive y coordinate 
                | * right
                |     The origin is on the y axis with a negative y coordinate 
                | * top
                |     The origin is on the z axis with a positive z coordinate 
                | * bottom
                |     The origin is on the z axis with a negative z coordinate 
                | 
                | The cameras of the Cameras collection are available using the dialog box
                | displayed by clicking the View->Defined Views menu.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Camera)
        self.cameras = com_object

    def item(self, i_index: cat_variant) -> Camera:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Item(CATVariant iIndex) As Camera
                | 
                |     Returns a camera using its index or its name from the Cameras
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the camera to retrieve from the collection
                |             of cameras. As a numerics, this index is the rank of the camera in the
                |             collection. The index of the first camera in the collection is 1, and the index
                |             of the last camera is Count. As a string, it is the name you assigned to the
                |             camera using the 
                | 
                |         AnyObject.Name property. 
                |     Returns:
                |         The retrieved camera 
                |     Example:
                |         This example retrieves in ThisCamera the ninth camera, and in
                |         ThatCamera the camera named MyCamera in the camera collection of the active
                |         document.
                | 
                |          Dim ThisCamera As Camera
                |          Set ThisCamera = CATIA.ActiveDocument.Cameras.Item(9)
                |          Dim ThatCamera As Camera
                |          Set ThatCamera = CATIA.ActiveDocument.Cameras.Item("MyCamera")

        :param cat_variant i_index:
        :return: Camera
        :rtype: Camera
        """
        return Camera(self.cameras.Item(i_index))

    def remove(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub Remove(CATVariant iIndex)
                | 
                |     Removes a camera from the Cameras collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the camera to remove from the collection
                |             of cameras. As a numerics, this index is the rank of the camera in the
                |             collection. The index of the first camera in the collection is 1, and the index
                |             of the last camera is Count. As a string, it is the name you assigned to the
                |             camera using the 
                | 
                |         AnyObject.Name property. You cannot remove the first seventh cameras in
                |         the collection. 
                | 
                | Example:
                |     The following example removes the tenth camera and the camera named
                |     CameraToBeRemoved in the camera collection of the active
                |     document.
                | 
                |      CATIA.ActiveDocument.Cameras.Remove(10)
                |      CATIA.ActiveDocument.Cameras.Remove("CameraToBeRemoved")

        :param cat_variant i_index:
        :return: None
        :rtype: None
        """
        return self.cameras.Remove(i_index)

    def __getitem__(self, n: int) -> Camera:
        if (n + 1) > self.count:
            raise StopIteration

        return Camera(self.cameras.item(n + 1))

    def __iter__(self) -> Iterator[Camera]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'Cameras(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject
from pycatia.tps_interfaces.associated_ref_frame import AssociatedRefFrame
from pycatia.tps_interfaces.composite_tolerance import CompositeTolerance
from pycatia.tps_interfaces.free_state import FreeState
from pycatia.tps_interfaces.material_condition import MaterialCondition
from pycatia.tps_interfaces.particular_tol_elem import ParticularTolElem
from pycatia.tps_interfaces.projected_tolerance_zone import ProjectedToleranceZone
from pycatia.tps_interfaces.shifted_profile_tolerance import ShiftedProfileTolerance
from pycatia.tps_interfaces.tps_parallel_on_screen import TPSParallelOnScreen
from pycatia.tps_interfaces.tangent_plane import TangentPlane
from pycatia.tps_interfaces.tolerance_per_unit_basis_restrictive_value import TolerancePerUnitBasisRestrictiveValue
from pycatia.tps_interfaces.tolerance_unit_basis_value import ToleranceUnitBasisValue
from pycatia.tps_interfaces.tolerance_zone import ToleranceZone


class SemanticGDT(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SemanticGDT
                | 
                | Interface managing Semantic GDT.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.semantic_gdt = com_object

    def associated_ref_frame(self) -> AssociatedRefFrame:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AssociatedRefFrame() As AssociatedRefFrame
                | 
                |     Returns the annotation on the AssociatedRefFrame
                |     interface.
                | 
                |     Parameters:
                | 
                |         oAssRefFra
                |             The AssociatedRefFrame.

        :return: AssociatedRefFrame
        :rtype: AssociatedRefFrame
        """
        return AssociatedRefFrame(self.semantic_gdt.AssociatedRefFrame())

    def composite_tolerance(self) -> CompositeTolerance:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func CompositeTolerance() As CompositeTolerance
                | 
                |     Returns the CompositeTolerance interface.
                | 
                |     Parameters:
                | 
                |         oCompTol
                |             The CompositeTolerance.

        :return: CompositeTolerance
        :rtype: CompositeTolerance
        """
        return CompositeTolerance(self.semantic_gdt.CompositeTolerance())

    def free_state(self) -> FreeState:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func FreeState() As FreeState
                | 
                |     Returns the FreeState interface.
                | 
                |     Parameters:
                | 
                |         oFreeState
                |             The FreeState.

        :return: FreeState
        :rtype: FreeState
        """
        return FreeState(self.semantic_gdt.FreeState())

    def has_a_free_state(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func HasAFreeState() As boolean
                | 
                |     Checks if the GDT has a Free State.
                | 
                |     Parameters:
                | 
                |         oHasAFreeState
                | 
                |                 TRUE: There is a Free State
                |                 FALSE: There is no Free State

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.HasAFreeState()

    def has_a_material_condition(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func HasAMaterialCondition() As boolean
                | 
                |     Checks if the GDT has a Material Condition.
                | 
                |     Parameters:
                | 
                |         oHasMatCond
                | 
                |                 TRUE: A Material Condition exits
                |                 FALSE: There is no Material Condition

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.HasAMaterialCondition()

    def has_a_particular_tol_elem(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func HasAParticularTolElem() As boolean
                | 
                |     Checks if the GDT has a Particuler Element.
                | 
                |     Parameters:
                | 
                |         oHasAParTolElem
                | 
                |                 TRUE: There is a Particuler Element
                |                 FALSE: There is no Particuler Element

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.HasAParticularTolElem()

    def has_a_tangent_plane(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func HasATangentPlane() As boolean
                | 
                |     Checks if the GDT has a Tangent Plane Modifier.
                | 
                |     Parameters:
                | 
                |         oIsATangentPlane
                | 
                |                 TRUE: There has a Tangent Plane
                |                 FALSE: There has not Tangent Plane

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.HasATangentPlane()

    def has_a_tolerance_per_unit_basis_restrictive_value(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func HasATolerancePerUnitBasisRestrictiveValue() As boolean
                | 
                |     Checks if the GDT has a Tolerance Per Unit Basis Restricted
                |     Value.
                | 
                |     Parameters:
                | 
                |         oHasATolRes
                | 
                |                 TRUE: There is a Tolerance Per Unit Basis Restricted
                |                 Value
                |                 FALSE: There is no Tolerance Per Unit Basis Restricted
                |                 Value

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.HasATolerancePerUnitBasisRestrictiveValue()

    def is_a_composite_tolerance(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsACompositeTolerance() As boolean
                | 
                |     Checks if the GDT is a Composite Tolerance.
                | 
                |     Parameters:
                | 
                |         oIsACompTol
                | 
                |                 TRUE: There is a Composite Tolerance
                |                 FALSE: There is no Composite Tolerance

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsACompositeTolerance()

    def is_a_projected_tolerance_zone(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsAProjectedToleranceZone() As boolean
                | 
                |     Checks if the GDT is a Projected Zone.
                | 
                |     Parameters:
                | 
                |         oIsAProjTolZone
                | 
                |                 TRUE: There is a Projected Zone
                |                 FALSE: There is no Projected Zone

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsAProjectedToleranceZone()

    def is_a_shifted_profile_tolerance(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsAShiftedProfileTolerance() As boolean
                | 
                |     Checks if the GDT is a Shifted Profile Tolerance.
                | 
                |     Parameters:
                | 
                |         oIsAShiftProTol
                | 
                |                 TRUE: There is a Shifted Profile Tolerance
                |                 FALSE: There is no Shifted Profile Tolerance

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsAShiftedProfileTolerance()

    def is_a_tolerance_unit_basis_value(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsAToleranceUnitBasisValue() As boolean
                | 
                |     Checks if the GDT is a Tolerance Unit Basis Value.
                | 
                |     Parameters:
                | 
                |         oIsATolUnitBas
                | 
                |                 TRUE: A Tolerance Zone exits
                |                 FALSE: There is no Tolerance Zone

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsAToleranceUnitBasisValue()

    def is_a_tolerance_zone(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsAToleranceZone() As boolean
                | 
                |     Checks if a Tolerance Zone exists.
                | 
                |     Parameters:
                | 
                |         oIsATolZone
                | 
                |                 TRUE: A Tolerance Zone exits
                |                 FALSE: There is no Tolerance Zone

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsAToleranceZone()

    def is_an_associated_ref_frame(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func IsAnAssociatedRefFrame() As boolean
                | 
                |     Checks if the GDT is an Associated Reference Frame.
                | 
                |     Parameters:
                | 
                |         oIsAnAssRefFra
                | 
                |                 TRUE: There is an Associated Reference Frame
                |                 FALSE: There is no Associated Reference Frame

        :return: bool
        :rtype: bool
        """
        return self.semantic_gdt.IsAnAssociatedRefFrame()

    def material_condition(self) -> MaterialCondition:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func MaterialCondition() As MaterialCondition
                | 
                |     Returns the MaterialCondition interface.
                | 
                |     Parameters:
                | 
                |         oMatCond
                |             The Material Condition.

        :return: MaterialCondition
        :rtype: MaterialCondition
        """
        return MaterialCondition(self.semantic_gdt.MaterialCondition())

    def particular_tol_elem(self) -> ParticularTolElem:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ParticularTolElem() As ParticularTolElem
                | 
                |     Returns the ParticularTolElem interface.
                | 
                |     Parameters:
                | 
                |         oParTolElem
                |             The ParticularTolElem.

        :return: ParticularTolElem
        :rtype: ParticularTolElem
        """
        return ParticularTolElem(self.semantic_gdt.ParticularTolElem())

    def projected_tolerance_zone(self) -> ProjectedToleranceZone:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ProjectedToleranceZone() As ProjectedToleranceZone
                | 
                |     Returrns the ProjectedToleranceZone interface.
                | 
                |     Parameters:
                | 
                |         oProjTolZone
                |             The ProjectedToleranceZone.

        :return: ProjectedToleranceZone
        :rtype: ProjectedToleranceZone
        """
        return ProjectedToleranceZone(self.semantic_gdt.ProjectedToleranceZone())

    def shifted_profile_tolerance(self) -> ShiftedProfileTolerance:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ShiftedProfileTolerance() As ShiftedProfileTolerance
                | 
                |     Returns the ShiftedProfileTolerance interface.
                | 
                |     Parameters:
                | 
                |         oShiftProTol
                |             The ShiftedProfileTolerance.

        :return: ShiftedProfileTolerance
        :rtype: ShiftedProfileTolerance
        """
        return ShiftedProfileTolerance(self.semantic_gdt.ShiftedProfileTolerance())

    def tps_parallel_on_screen(self) -> TPSParallelOnScreen:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func TPSParallelOnScreen() As TPSParallelOnScreen
                | 
                |     Gets the annotation on TPSParallelOnScreen interface.

        :return: TPSParallelOnScreen
        :rtype: TPSParallelOnScreen
        """
        return TPSParallelOnScreen(self.semantic_gdt.TPSParallelOnScreen())

    def tangent_plane(self) -> TangentPlane:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func TangentPlane() As TangentPlane
                | 
                |     Returns the Tangent Plane interface.
                | 
                |     Parameters:
                | 
                |         oTangentPlane
                |             The Tangent Plane.

        :return: TangentPlane
        :rtype: TangentPlane
        """
        return TangentPlane(self.semantic_gdt.TangentPlane())

    def tolerance_per_unit_basis_restrictive_value(self) -> TolerancePerUnitBasisRestrictiveValue:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func TolerancePerUnitBasisRestrictiveValue() As
                | TolerancePerUnitBasisRestrictiveValue
                | 
                |     Returns the TolerancePerUnitBasisRestrictiveValue
                |     interface.
                | 
                |     Parameters:
                | 
                |         oTolRes
                |             The TolerancePerUnitBasisRestrictiveValue.

        :return: TolerancePerUnitBasisRestrictiveValue
        :rtype: TolerancePerUnitBasisRestrictiveValue
        """
        return TolerancePerUnitBasisRestrictiveValue(self.semantic_gdt.TolerancePerUnitBasisRestrictiveValue())

    def tolerance_unit_basis_value(self) -> ToleranceUnitBasisValue:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ToleranceUnitBasisValue() As ToleranceUnitBasisValue
                | 
                |     Returns the ToleranceUnitBasisValue interface.
                | 
                |     Parameters:
                | 
                |         oTolUnitBas
                |             The Tolerance Unit Basis Value.

        :return: ToleranceUnitBasisValue
        :rtype: ToleranceUnitBasisValue
        """
        return ToleranceUnitBasisValue(self.semantic_gdt.ToleranceUnitBasisValue())

    def tolerance_zone(self) -> ToleranceZone:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ToleranceZone() As ToleranceZone
                | 
                |     Returns the ToleranceZone interface.
                | 
                |     Parameters:
                | 
                |         oTolZone
                |             The Tolerance Zone.

        :return: ToleranceZone
        :rtype: ToleranceZone
        """
        return ToleranceZone(self.semantic_gdt.ToleranceZone())

    def __repr__(self):
        return f'SemanticGdt(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.drafting_interfaces.drawing_text import DrawingText
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.tps_interfaces.tps_parallel_on_screen import TPSParallelOnScreen


class Text(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Text
                | 
                | Interface for the TPS Text object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.text_com = com_object

    @property
    def text(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Text() As CATBSTR
                | 
                |     Gets the annotation's text.

        :return: str
        :rtype: str
        """

        return self.text_com.Text

    @text.setter
    def text(self, value: str):
        """
        :param str value:
        """

        self.text_com.Text = value

    def get_2d_annot(self) -> DrawingText:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Get2dAnnot() As DrawingText
                | 
                |     Retrieves Drafting text.

        :return: DrawingText
        :rtype: DrawingText
        """
        return DrawingText(self.text_com.Get2dAnnot())

    def tps_parallel_on_screen(self) -> TPSParallelOnScreen:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func TPSParallelOnScreen() As TPSParallelOnScreen
                | 
                |     Gets the annotation on TPSParallelOnScreen interface.

        :return: TPSParallelOnScreen
        :rtype: TPSParallelOnScreen
        """
        return TPSParallelOnScreen(self.text_com.TPSParallelOnScreen())

    def __repr__(self):
        return f'Text(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.real_param import RealParam
from pycatia.knowledge_interfaces.unit import Unit


class Dimension(RealParam):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     KnowledgeInterfaces.Parameter
                |                         KnowledgeInterfaces.RealParam
                |                             Dimension
                | 
                | Represents the dimension parameter.
                | It is an abstract object which is not intended to be created as such, but from
                | which the length and angle parameters derive.
                | 
                | See also:
                |     Length, Angle
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.dimension = com_object

    @property
    def unit(self) -> Unit:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Unit() As Unit (Read Only)
                | 
                |     Returns the unit used for this dimension object.

        :return: Unit
        :rtype: Unit
        """

        return Unit(self.dimension.Unit)

    def value_as_string2(self, i_nb_decimals: int, i_show_trailing_zeros: bool) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func ValueAsString2(long iNbDecimals,
                | boolean iShowTrailingZeros) As CATBSTR
                | 
                |     Gets the value of the parameter as a string, with a given
                |     precision.
                | 
                |     Parameters:
                | 
                |         iNbDecimals
                |             the maximum number of decimal places to use to generate the string
                |             (minimum 0, maximum 9) 
                |         iShowTrailingZeros
                |             this argument says if trailing zeros have to be shown
                |             
                | 
                |     Example:
                |         This example gets the value of the existing dimension parameter and
                |         shows it in a message box
                | 
                |          Dim str
                |          str = dimension.ValueAsString;
                |          MessageBox str

        :param int i_nb_decimals:
        :param bool i_show_trailing_zeros:
        :return: str
        :rtype: str
        """
        return self.dimension.ValueAsString2(i_nb_decimals, i_show_trailing_zeros)

    def __repr__(self):
        return f'Dimension(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.drafting_interfaces.drawing_welding import DrawingWelding
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.tps_interfaces.tps_parallel_on_screen import TPSParallelOnScreen


class Weld(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Weld

    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.weld = com_object

    def get_2d_annot(self) -> DrawingWelding:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Get2dAnnot() As DrawingWelding
                | 
                |     Retrieves Drafting Weld annotation.

        :return: DrawingWelding
        :rtype: DrawingWelding
        """
        return DrawingWelding(self.weld.Get2dAnnot())

    def tps_parallel_on_screen(self) -> TPSParallelOnScreen:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func TPSParallelOnScreen() As TPSParallelOnScreen
                | 
                |     Gets the annotation on TPSParallelOnScreen interface.

        :return: TPSParallelOnScreen
        :rtype: TPSParallelOnScreen
        """
        return TPSParallelOnScreen(self.weld.TPSParallelOnScreen())

    def __repr__(self):
        return f'Weld(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.relation import Relation


class Rule(Relation):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     KnowledgeInterfaces.KnowledgeObject
                |                        KnowledgeInterfaces.KnowledgeActivateObject
                |                             KnowledgeInterfaces.Relation
                |                                 Rule
                | 
                | Represents the program relation.
                | The following example shows how to create a program that selects a depth with
                | respect to a mass value. The depth and mass parameters should exist before the
                | creation of the program (also called Rule) object.
                | 
                | 	Dim CATDocs As Documents
                |  Set CATDocs = CATIA.Documents
                |  Dim part1 As Document
                |  Set part1   = CATDocs.Add("CATPart")
                |  Dim mass As RealParam
                |  Set mass         = part1.Part.Parameters.CreateReal("mass", 5.)
                |  Dim depth As RealParam
                |  Set depth        = part1.Part.Parameters.CreateReal("depth", 0.)
                |  Dim selectdepth As Relation
                |  Set selectdepth = part1.Part.Relations.CreateProgram
                |                     ("select_depth",
                |                      "Select depth with respect to mass", 
                |                      "if (mass>2kg) { depth=2mm } else { depth=1mm
                |                      }")
                |  
                | 
                | Copyright © 1999-2011, Dassault Systèmes. All rights reserved.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.rule = com_object

    def __repr__(self):
        return f'Rule(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeExtrapol(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeExtrapol
                | 
                | Represents the hybrid shape extrapolation feature object.
                | Role: To access the data of the hybrid shape affinity feature object. The
                | hybrid shape extrapolation feature object is created by using an element (a
                | curve or a surface), a boundary of this element (a point in case of curve
                | extrapolation or a curve in case of surface extrapolation), and a limit (which
                | can be specified by a length or a limit element).
                | The continuity between the extrapolated element and the extrapolation can be
                | either tangent continuity or curvature continuity.
                | The extrapolation can be assembled or not with the extrapolated curve or
                | surface. In case of surface extrapolation, extrapolation borders can
                | be:
                | 
                |     Normal to the boundary of the extrapolated surface
                |     Tangent to the edges of the extrapolated surface, that are adjacent to the
                |     boundary
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeExtrapol
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_extrapol = com_object

    @property
    def border_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property BorderType() As long
                | 
                |     Returns or sets the border type of extrapolation.
                |     This applies for surface extrapolation only.
                |     Legal values: the border type is either normal to the boundary of the
                |     extrapolated surface (CATGSMNormalBorder(=0)), or tangent to the edges of the
                |     extrapolated surface that are adjacent to the boundary
                |     CATGSMTangentBorder(=1)).

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_extrapol.BorderType

    @border_type.setter
    def border_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_extrapol.BorderType = value

    @property
    def boundary(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Boundary() As Reference
                | 
                |     This method is left available for compatibility reasons but it is
                |     preferable to use the GetBoundary method instead.
                | 
                |     Migration instructions: use the signature that uses an integer as the first
                |     argument.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_extrapol.Boundary)

    @boundary.setter
    def boundary(self, reference_boundary: Reference):
        """
        :param Reference reference_boundary:
        """

        self.hybrid_shape_extrapol.Boundary = reference_boundary.com_object

    @property
    def constant_length_mode(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ConstantLengthMode() As boolean
                | 
                |     Returns or sets the constant distance mode in case of Length extrapolation
                |     limit.
                |     This applies in case of Length extrapolation limit.

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_extrapol.ConstantLengthMode

    @constant_length_mode.setter
    def constant_length_mode(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_extrapol.ConstantLengthMode = value

    @property
    def continuity_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ContinuityType() As long
                | 
                |     This method is left available for compatibility reasons but it is
                |     preferable to use the GetContinuityType method instead.
                | 
                |     Migration instructions: use the signature that uses an integer as the first
                |     argument.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_extrapol.ContinuityType

    @continuity_type.setter
    def continuity_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_extrapol.ContinuityType = value

    @property
    def elem_to_extrapol(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ElemToExtrapol() As Reference
                | 
                |     Returns or sets the curve or surface to extrapolate.
                |     Sub-element(s) supported (see Boundary object): see Face , TriDimFeatEdge
                |     or BiDimFeatEdge.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_extrapol.ElemToExtrapol)

    @elem_to_extrapol.setter
    def elem_to_extrapol(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_extrapol.ElemToExtrapol = reference_element.com_object

    @property
    def elem_until(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ElemUntil() As Reference
                | 
                |     This method is left available for compatibility reasons but it is
                |     preferable to use the GetElemUntil method instead.
                | 
                |     Migration instructions: use the signature that uses an integer as the first
                |     argument.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_extrapol.ElemUntil)

    @elem_until.setter
    def elem_until(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_extrapol.ElemUntil = reference_element.com_object

    @property
    def extend_edges_mode(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ExtendEdgesMode() As boolean
                | 
                |     Returns or sets the extension of extrapolated edges mode.
                |     This applies in case of tangent continuity mode, tangent border mode and
                |     assembled result.

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_extrapol.ExtendEdgesMode

    @extend_edges_mode.setter
    def extend_edges_mode(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_extrapol.ExtendEdgesMode = value

    @property
    def extrapol_both_sides_identically(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ExtrapolBothSidesIdentically() As boolean
                | 
                |     Returns or sets the the boolean telling if the second side is extrapolated
                |     according to the first side's settings or to its own ones.
                |     This applies in case the element to extrapol is a wire.

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_extrapol.ExtrapolBothSidesIdentically

    @extrapol_both_sides_identically.setter
    def extrapol_both_sides_identically(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_extrapol.ExtrapolBothSidesIdentically = value

    @property
    def length(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Length() As Length (Read Only)
                | 
                |     This method is left available for compatibility reasons but it is
                |     preferable to use the GetLength method instead.
                | 
                |     Migration instructions: use the signature that uses an integer as the first
                |     argument.

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_extrapol.Length)

    @property
    def limit_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property LimitType() As long
                | 
                |     This method is left available for compatibility reasons but it is
                |     preferable to use the GetLimitType method instead.
                | 
                |     Migration instructions: use the signature that uses an integer as the first
                |     argument.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_extrapol.LimitType

    @limit_type.setter
    def limit_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_extrapol.LimitType = value

    @property
    def propagation_mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PropagationMode() As long
                | 
                |     Returns or sets the propagation mode.
                |     This applies in case of curvature extrapolation of a shell.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_extrapol.PropagationMode

    @propagation_mode.setter
    def propagation_mode(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_extrapol.PropagationMode = value

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Returns or sets the support surface.
                |     This applies in case of tangent extrapolation of a wire. If a support
                |     surface is given, the extrapolation will lie on it.
                |     Sub-element(s) supported (see Boundary object): see Face.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_extrapol.Support)

    @support.setter
    def support(self, value: Reference):
        """
        :param Reference value:
        """

        self.hybrid_shape_extrapol.Support = value

    def get_boundary(self, i_pos: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetBoundary(long iPos) As Reference
                | 
                |     Returns or sets the iPos-th boundary of an extrapolated curve or surface
                |     from which extrapolation begins. If iPos equals 0 or the number of
                |     extrapolations, SetBoundary appends iBoundary to the list of
                |     boundaries.
                |     The boudary is a point for an extrapolated curve, or a curve for an
                |     extrapolated surface.
                |     Sub-element(s) supported (see Boundary object): see Face , TriDimFeatEdge
                |     or BiDimFeatEdge.

        :param int i_pos:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.hybrid_shape_extrapol.GetBoundary(i_pos))

    def get_continuity_type(self, i_pos: int) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetContinuityType(long iPos) As long
                | 
                |     Returns or sets the continuity type between extrapolated element and
                |     extrapolation at the iPos-th boundary. If iPos equals 0 or the number of
                |     extrapolations, SetContinuityType appends iLim to the list of
                |     lengths.
                |     Legal values: the continuity type is either CATGSMTangentContinuity (=0) or
                |     CATGSMCurvatureContinuity (=1).

        :param int i_pos:
        :return: int
        :rtype: int
        """
        return self.hybrid_shape_extrapol.GetContinuityType(i_pos)

    def get_elem_until(self, i_pos: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetElemUntil(long iPos) As Reference
                | 
                |     Returns or sets the surface or volume specifying the limit of the
                |     extrapolation that begins from the iPos-th boundary. If iPos equals 0 or the
                |     number of extrapolations, SetElemUntil appends iElemUntil to the list of up-to
                |     elements.
                |     This applies when the limit type is CATGSMUpToElementLimit (=1).

        :param int i_pos:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.hybrid_shape_extrapol.GetElemUntil(i_pos))

    def get_internal_edges_element(self, i_pos: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetInternalEdgesElement(long iPos) As Reference
                | 
                |     Gets an element in the list of internal elements (vertex or
                |     edges).
                | 
                |     Parameters:
                | 
                |         oInternalElement
                |             internal element 
                |         iPos
                |             position of internal element to be retrieved.

        :param int i_pos:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.hybrid_shape_extrapol.GetInternalEdgesElement(i_pos))

    def get_length(self, i_pos: int) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetLength(long iPos) As Length
                | 
                |     Returns or sets the length specifying the limit of the extrapolation that
                |     begins from the iPos-th boundary. If iPos equals 0 or the number of
                |     extrapolations, SetLength appends iLength to the list of
                |     lengths.
                |     This applies when the limit type is CATGSMLengthLimit (=0).

        :param int i_pos:
        :return: Length
        :rtype: Length
        """
        return Length(self.hybrid_shape_extrapol.GetLength(i_pos))

    def get_limit_type(self, i_pos: int) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetLimitType(long iPos) As long
                | 
                |     Returns or sets the limit type of the extrapolation that begins from the
                |     iPos-th boundary. If iPos equals 0 or the number of extrapolations,
                |     SetLimitType appends iLim to the list of limit types.
                |     The limit can be a length, a surface, or a volume.
                |     Legal values: the limit type is either CATGSMLengthLimit(0) or
                |     CATGSMUpToElementLimit(1).

        :param int i_pos:
        :return: int
        :rtype: int
        """
        return self.hybrid_shape_extrapol.GetLimitType(i_pos)

    def get_number_of_extrapolations(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetNumberOfExtrapolations(long
                | oNumberOfExtrapolations)
                | 
                |     Returns the number of extrapolations set. An extrapolation is specified by
                |     a boundary of the element to extrapolate, a limit type, a limit and a
                |     continuity type.

        :return: int
        :rtype: int
        """
        return self.hybrid_shape_extrapol.GetNumberOfExtrapolations()

    def is_assemble(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func IsAssemble() As boolean
                | 
                |     Retrieves whether extrapolation is assembled with extrapolated curve or
                |     surface.
                | 
                |     Parameters:
                | 
                |         oAssemble
                |             The assemble option
                |             True when the extrapolation is assembled with extrapolated curve or
                |             surface, and False otherwise

        :return: bool
        :rtype: bool
        """
        return self.hybrid_shape_extrapol.IsAssemble()

    def remove_all_extrapolations_except_the_first_one(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveAllExtrapolationsExceptTheFirstOne()
                | 
                |     Removes all extrapolations that may have been set, except the first one.

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.RemoveAllExtrapolationsExceptTheFirstOne()

    def remove_all_internal_edges_element(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveAllInternalEdgesElement()
                | 
                |     Removes all internal elements.

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.RemoveAllInternalEdgesElement()

    def remove_extrapolation(self, i_pos: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveExtrapolation(long iPos)
                | 
                |     Removes the iPos-th extrapolation that has been set.

        :param int i_pos:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.RemoveExtrapolation(i_pos)

    def set_assemble(self, i_assemble: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetAssemble(boolean iAssemble)
                | 
                |     Sets whether extrapolation is to be assembled with extrapolated curve or
                |     surface.
                | 
                |     Parameters:
                | 
                |         iAssemble
                |             The assemble option
                |             True when the extrapolation is to be assembled with extrapolated
                |             curve or surface, and False otherwise.

        :param bool i_assemble:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetAssemble(i_assemble)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_assemble'
        # # vba_code = """
        # # Public Function set_assemble(hybrid_shape_extrapol)
        # #     Dim iAssemble (2)
        # #     hybrid_shape_extrapol.SetAssemble iAssemble
        # #     set_assemble = iAssemble
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_boundary(self, i_pos: int, i_boundary: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetBoundary(long iPos,
                | Reference iBoundary)

        :param int i_pos:
        :param Reference i_boundary:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetBoundary(i_pos, i_boundary.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_boundary'
        # # vba_code = """
        # # Public Function set_boundary(hybrid_shape_extrapol)
        # #     Dim iPos (2)
        # #     hybrid_shape_extrapol.SetBoundary iPos
        # #     set_boundary = iPos
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_continuity_type(self, i_pos: int, i_lim: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetContinuityType(long iPos,
                | long iLim)

        :param int i_pos:
        :param int i_lim:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetContinuityType(i_pos, i_lim)

    def set_elem_until(self, i_pos: int, i_elem_until: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetElemUntil(long iPos,
                | Reference iElemUntil)

        :param int i_pos:
        :param Reference i_elem_until:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetElemUntil(i_pos, i_elem_until.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_elem_until'
        # # vba_code = """
        # # Public Function set_elem_until(hybrid_shape_extrapol)
        # #     Dim iPos (2)
        # #     hybrid_shape_extrapol.SetElemUntil iPos
        # #     set_elem_until = iPos
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_length(self, i_pos: int, i_length: Length) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLength(long iPos,
                | Length iLength)
                | 
                |     Do not use this method.

        :param int i_pos:
        :param Length i_length:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetLength(i_pos, i_length.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_length'
        # # vba_code = """
        # # Public Function set_length(hybrid_shape_extrapol)
        # #     Dim iPos (2)
        # #     hybrid_shape_extrapol.SetLength iPos
        # #     set_length = iPos
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_length_d(self, i_pos: int, i_length: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLengthD(long iPos,
                | double iLength)

        :param int i_pos:
        :param float i_length:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetLengthD(i_pos, i_length)

    def set_limit_type(self, i_pos: int, i_lim: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLimitType(long iPos,
                | long iLim)

        :param int i_pos:
        :param int i_lim:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_extrapol.SetLimitType(i_pos, i_lim)

    def __repr__(self):
        return f'HybridShapeExtrapol(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.real_param import RealParam
from pycatia.system_interfaces.any_object import AnyObject


class FreeParameter(AnyObject):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     FreeParameter
                | 
                | Interface to access a CATIAFreeParameter.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.free_parameter = com_object

    @property
    def inf_range(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property InfRange() As double
                | 
                |     Returns or sets the inferior bound of the free parameter object. The
                |     optimization cannot escape those bounds.

        :return: float
        :rtype: float
        """

        return self.free_parameter.InfRange

    @inf_range.setter
    def inf_range(self, value: float):
        """
        :param float value:
        """

        self.free_parameter.InfRange = value

    @property
    def parameter(self) -> RealParam:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Parameter() As RealParam
                | 
                |     Returns or sets which parameter (CATIAParameter) is linked to this object.
                |     The parameter must be real.

        :return: RealParam
        :rtype: RealParam
        """

        return RealParam(self.free_parameter.Parameter)

    @parameter.setter
    def parameter(self, value: RealParam):
        """
        :param RealParam value:
        """

        self.free_parameter.Parameter = value

    @property
    def step(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Step() As double
                | 
                |     Returns or sets the initial step used by the optimisation to look for a
                |     better solution. This step is just a preliminary indication. It will vary
                |     during the optimisation process.

        :return: float
        :rtype: float
        """

        return self.free_parameter.Step

    @step.setter
    def step(self, value: float):
        """
        :param float value:
        """

        self.free_parameter.Step = value

    @property
    def sup_range(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property SupRange() As double
                | 
                |     Returns or sets the superior bound of the free parameter object. The
                |     optimization cannot escape those bounds.

        :return: float
        :rtype: float
        """

        return self.free_parameter.SupRange

    @sup_range.setter
    def sup_range(self, value: float):
        """
        :param float value:
        """

        self.free_parameter.SupRange = value

    def __repr__(self):
        return f'FreeParameter(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject


class Camera(AnyObject):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Camera
                | 
                | Represents the camera.
                | The camera is the object that stores a viewpoint saved from a viewer at a given
                | moment using the Viewer.NewCamera method of the Viewer object. The viewpoint
                | stored in the camera can then be applied to another viewer to display the
                | document in this viewer according to this viewpoint.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.camera = com_object

    @property
    def type(self):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Type() As CatCameraType (Read Only)
                | 
                |     Returns the camera's type.
                | 
                |     Example:
                |         This example retrieves in MyCameraType the type of the MyCamera 3D
                |         camera and applies the viewpoint stored in this camera to the active
                |         viewer.
                | 
                |          MyCameraType = MyCamera.Type
                |          CATIA.ActiveWindow.ActiveViewer.Viewpoint3D = MyCamera.Viewpoint3D
                |          
                | 
                |         The value returned by the Type property in MyCameraType is catCamera3D

        :return: enum cat_camera_type
        """

        return self.camera.Type

    def __repr__(self):
        return f'Camera(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.set_of_equation import SetOfEquation


class ConstraintSatisfaction(SetOfEquation):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     KnowledgeInterfaces.KnowledgeObject
                |                        KnowledgeInterfaces.KnowledgeActivateObject
                |                             KnowledgeInterfaces.Relation
                |                                KnowledgeInterfaces.SetOfEquation
                |                                     ConstraintSatisfaction
                | 
                | Represents the set of equation.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.constraint_satisfaction = com_object

    def solve(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Solve()
                | 
                |     Solves the constraint satisfaction problem.

        :return: None
        :rtype: None
        """
        return self.constraint_satisfaction.Solve()

    def __repr__(self):
        return f'ConstraintSatisfaction(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from pycatia.mec_mod_interfaces.boundary import Boundary


class Edge(Boundary):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     InfInterfaces.Reference
                |                         MecModInterfaces.Boundary
                |                             Edge
                |
                | 1-D boundary.
                | Role: This Boundary object may be, for example, the edge of a
                | cylinder.
                | You will create an Edge object using the Shapes.GetBoundary ,
                | HybridShapes.GetBoundary , Sketches.GetBoundary or Selection.SelectElement2
                | method. Then, you pass it to the operator (such as
                | HybridShapeFactory.AddNewPointTangent ).
                | The lifetime of an Edge object is limited, see Boundary.
                | See also:
                | TriDimFeatEdge , RectilinearTriDimFeatEdge , BiDimFeatEdge ,
                | RectilinearBiDimFeatEdge , MonoDimFeatEdge , RectilinearMonoDimFeatEdge
                | .
                |
                | Example:
                |     This example asks the end user to select a planar curve, whose plane is
                |     parallel to the XY plane. Then, it creates a point on the tangent to the curve
                |     in the X direction:
                |
                |      Dim InputObjectType(0)
                |      Set Document = CATIA.ActiveDocument
                |      Set Selection = Document.Selection
                |      Set HybridBodies = Document.Part.HybridBodies
                |      Set HybridBody = HybridBodies.Item("Geometrical Set.1")
                |      'We propose to the user that he select a planar curve whose plane is
                |      parallel to the XY plane
                |      InputObjectType(0)="Edge"
                |      Status=Selection.SelectElement2(InputObjectType,"Select a planar curve
                |      whose plane is parallel to the XY plane",true)
                |      if (Status = "cancel") then Exit Sub
                |      Set PlanarCurve = Selection.Item(1).Value
                |      Set HybridShapeDirection = HybridShapeFactory.AddNewDirectionByCoord(1.0,0.0,0.0)
                |      Set HybridShapePointTangent = HybridShapeFactory.
                |                                        AddNewPointTangent(PlanarCurve,HybridShapeDirection)
                |      HybridBody.AppendHybridShape HybridShapePointTangent
                |      Document.Part.InWorkObject = HybridShapePointTangent
                |      Document.Part.Update

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.edge = com_object

    def __repr__(self):
        return f'Edge(name="{self.name}")'
